CursorStyle

Enum CursorStyle 

pub enum CursorStyle {
Show 15 variants Default, Pointer, Text, Crosshair, Move, NotAllowed, ResizeNS, ResizeEW, ResizeNESW, ResizeNWSE, Grab, Grabbing, Wait, Progress, None,
}
Mouse cursor style for an element

When the cursor hovers over an element with a cursor style set, the window cursor will change to this style.

Variants



Default

Default arrow cursor



Pointer

Pointer/hand cursor (for clickable elements like links, buttons)



Text

Text/I-beam cursor (for text input)



Crosshair

Crosshair cursor



Move

Move cursor (for dragging)



NotAllowed

Not allowed cursor



ResizeNS

North-South resize cursor



ResizeEW

East-West resize cursor



ResizeNESW

Northeast-Southwest resize cursor



ResizeNWSE

Northwest-Southeast resize cursor



Grab

Grab cursor (open hand)



Grabbing

Grabbing cursor (closed hand)



Wait

Wait/loading cursor



Progress

Progress cursor (arrow with spinner)



None

Hidden cursor