BoundStateful

Struct BoundStateful 

pub struct BoundStateful<S>{ /* private fields */ }
A bound stateful element that maintains shared storage with an ElementRef

This wrapper is returned by Stateful::bind() and provides the same fluent API as Stateful, but all modifications go through shared storage accessible via the original ElementRef.

Implementations

Source

impl<S> BoundStateful<S>

pub fn on_state<F>(self, callback: F) -> BoundStateful<S>
where F: Fn(&S, &mut Div) + Send + Sync + 'static,

Set the state callback (builder pattern)

pub fn w(self, px: f32) -> BoundStateful<S>

Set width (builder pattern)

pub fn h(self, px: f32) -> BoundStateful<S>

Set height (builder pattern)

pub fn w_full(self) -> BoundStateful<S>

Set width to 100% (builder pattern)

pub fn h_full(self) -> BoundStateful<S>

Set height to 100% (builder pattern)

pub fn size(self, w: f32, h: f32) -> BoundStateful<S>

Set both width and height (builder pattern)

pub fn square(self, size: f32) -> BoundStateful<S>

Set square size (builder pattern)

pub fn flex_row(self) -> BoundStateful<S>

Set flex direction to row (builder pattern)

pub fn flex_col(self) -> BoundStateful<S>

Set flex direction to column (builder pattern)

pub fn flex_grow(self) -> BoundStateful<S>

Set flex grow (builder pattern)

pub fn w_fit(self) -> BoundStateful<S>

Set width to fit content (builder pattern)

pub fn h_fit(self) -> BoundStateful<S>

Set height to fit content (builder pattern)

pub fn p(self, units: f32) -> BoundStateful<S>

Set padding all sides (builder pattern)

pub fn px(self, units: f32) -> BoundStateful<S>

Set horizontal padding (builder pattern)

pub fn py(self, units: f32) -> BoundStateful<S>

Set vertical padding (builder pattern)

pub fn gap(self, units: f32) -> BoundStateful<S>

Set gap (builder pattern)

pub fn items_center(self) -> BoundStateful<S>

Center items (builder pattern)

pub fn justify_center(self) -> BoundStateful<S>

Center justify (builder pattern)

pub fn justify_between(self) -> BoundStateful<S>

Space between (builder pattern)

pub fn bg(self, color: impl Into<Brush>) -> BoundStateful<S>

Set background (builder pattern)

pub fn rounded(self, radius: f32) -> BoundStateful<S>

Set corner radius (builder pattern)

pub fn border(self, width: f32, color: Color) -> BoundStateful<S>

Set border with color and width (builder pattern)

pub fn border_color(self, color: Color) -> BoundStateful<S>

Set border color only (builder pattern)

pub fn border_width(self, width: f32) -> BoundStateful<S>

Set border width only (builder pattern)

pub fn shadow(self, shadow: Shadow) -> BoundStateful<S>

Set shadow (builder pattern)

pub fn shadow_sm(self) -> BoundStateful<S>

Set small shadow (builder pattern)

pub fn shadow_md(self) -> BoundStateful<S>

Set medium shadow (builder pattern)

pub fn shadow_lg(self) -> BoundStateful<S>

Set large shadow (builder pattern)

pub fn shadow_xl(self) -> BoundStateful<S>

Set extra-large shadow (builder pattern)

pub fn opacity(self, opacity: f32) -> BoundStateful<S>

Set opacity (builder pattern)

pub fn flex_shrink(self) -> BoundStateful<S>

Set flex shrink (builder pattern)

pub fn flex_shrink_0(self) -> BoundStateful<S>

Set flex shrink to 0 (builder pattern)

pub fn transform_style(self, xform: Transform) -> BoundStateful<S>

Set transform (builder pattern)

pub fn overflow_clip(self) -> BoundStateful<S>

Set overflow to clip (clips children to container bounds)

pub fn cursor(self, cursor: CursorStyle) -> BoundStateful<S>

Set cursor style (builder pattern)

pub fn cursor_pointer(self) -> BoundStateful<S>

Set cursor to pointer (hand) - convenience for clickable elements

pub fn cursor_text(self) -> BoundStateful<S>

Set cursor to text (I-beam) - for text input areas

pub fn child(self, child: impl ElementBuilder + 'static) -> BoundStateful<S>

Add child (builder pattern)

pub fn on_click<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a click handler (builder pattern)

pub fn on_mouse_down<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a mouse down handler (builder pattern)

pub fn on_mouse_up<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a mouse up handler (builder pattern)

pub fn on_hover_enter<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a hover enter handler (builder pattern)

pub fn on_hover_leave<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a hover leave handler (builder pattern)

pub fn on_focus<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a focus handler (builder pattern)

pub fn on_blur<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a blur handler (builder pattern)

pub fn on_mount<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a mount handler (builder pattern)

pub fn on_unmount<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register an unmount handler (builder pattern)

pub fn on_key_down<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a key down handler (builder pattern)

pub fn on_key_up<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a key up handler (builder pattern)

pub fn on_scroll<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a scroll handler (builder pattern)

pub fn on_mouse_move<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a mouse move handler (builder pattern)

pub fn on_resize<F>(self, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a resize handler (builder pattern)

pub fn on_event<F>(self, event_type: u32, handler: F) -> BoundStateful<S>
where F: Fn(&EventContext) + Send + Sync + 'static,

Register a handler for a specific event type (builder pattern)

pub fn on_layout<F>(self, callback: F) -> BoundStateful<S>
where F: Fn(ElementBounds) + Send + Sync + 'static,

Set a layout callback that fires synchronously after each layout computation (builder pattern)