pub struct FontFamily {
pub name: Option<String>,
pub generic: GenericFont,
}Expand description
Font family specification
Allows specifying either a named font (e.g., “Fira Code”, “Inter”) or a generic category. When a named font is specified, the generic category serves as a fallback if the font isn’t available.
Example
ⓘ
// Use a specific font with monospace fallback
text("code").font("Fira Code")
// Use system monospace
text("code").monospace()Fields
name: Option<String>Specific font name (e.g., “Fira Code”, “Inter”, “SF Pro”)
generic: GenericFontGeneric fallback category
Implementations
Sourceimpl FontFamily
impl FontFamily
Sourcepub fn generic(generic: GenericFont) -> FontFamily
pub fn generic(generic: GenericFont) -> FontFamily
Create a font family with just a generic category
Sourcepub fn named(name: impl Into<String>) -> FontFamily
pub fn named(name: impl Into<String>) -> FontFamily
Create a font family with a specific font name
Sourcepub fn named_with_fallback(
name: impl Into<String>,
generic: GenericFont,
) -> FontFamily
pub fn named_with_fallback( name: impl Into<String>, generic: GenericFont, ) -> FontFamily
Create a font family with a specific name and fallback category
Sourcepub fn system() -> FontFamily
pub fn system() -> FontFamily
System UI font
Sourcepub fn monospace() -> FontFamily
pub fn monospace() -> FontFamily
Monospace font
Sourcepub fn serif() -> FontFamily
pub fn serif() -> FontFamily
Serif font
Sourcepub fn sans_serif() -> FontFamily
pub fn sans_serif() -> FontFamily
Sans-serif font