ToastBuilder

Struct ToastBuilder 

pub struct ToastBuilder { /* private fields */ }
Builder for toast overlays

Implementations

Source

impl ToastBuilder

pub fn duration_ms(self, ms: u32) -> ToastBuilder

Set auto-dismiss duration in milliseconds

pub fn corner(self, corner: Corner) -> ToastBuilder

Set corner position

pub fn content<F>(self, f: F) -> ToastBuilder
where F: Fn() -> Div + Send + Sync + 'static,

Set the content using a builder function

pub fn motion_key(self, key: impl Into<String>) -> ToastBuilder

Set the motion key for triggering content exit animations

When the overlay transitions to Closing state, it will automatically trigger exit animation on the motion with this key. Use the same key with motion_derived(key) in your content builder.

pub fn show(self) -> OverlayHandle

Show the toast