RenderLayer

Enum RenderLayer 

pub enum RenderLayer {
    Background,
    Glass,
    Foreground,
}
Render layer for separating elements in glass-effect rendering

When using glass/vibrancy effects, elements need to be rendered in different passes:

  • Background elements are rendered first and get blurred behind glass
  • Glass elements render the glass effect itself
  • Foreground elements render on top without being blurred

Variants



Background

Rendered behind glass (will be blurred)



Glass

Rendered as a glass element (blur effect applied)



Foreground

Rendered on top of glass (not blurred)