pub struct SchedulerHandle { /* private fields */ }Expand description
A weak handle to the animation scheduler
This is passed to components that need to register animations. It won’t prevent the scheduler from being dropped.
Implementations
Sourceimpl SchedulerHandle
impl SchedulerHandle
Sourcepub fn register_spring(&self, spring: Spring) -> Option<SpringId>
pub fn register_spring(&self, spring: Spring) -> Option<SpringId>
Register a spring and return its ID
Sourcepub fn set_spring_target(&self, id: SpringId, target: f32)
pub fn set_spring_target(&self, id: SpringId, target: f32)
Update a spring’s target
Sourcepub fn get_spring_value(&self, id: SpringId) -> Option<f32>
pub fn get_spring_value(&self, id: SpringId) -> Option<f32>
Get current spring value
Sourcepub fn is_spring_settled(&self, id: SpringId) -> bool
pub fn is_spring_settled(&self, id: SpringId) -> bool
Check if a spring has settled (at rest at target)
Returns true if the spring exists and has settled, or if the spring
doesn’t exist (considered settled since there’s nothing animating).
Sourcepub fn remove_spring(&self, id: SpringId)
pub fn remove_spring(&self, id: SpringId)
Remove a spring
Sourcepub fn register_keyframe(
&self,
keyframe: KeyframeAnimation,
) -> Option<KeyframeId>
pub fn register_keyframe( &self, keyframe: KeyframeAnimation, ) -> Option<KeyframeId>
Register a keyframe animation and return its ID
Sourcepub fn get_keyframe_value(&self, id: KeyframeId) -> Option<f32>
pub fn get_keyframe_value(&self, id: KeyframeId) -> Option<f32>
Get current keyframe animation value
Sourcepub fn get_keyframe_progress(&self, id: KeyframeId) -> Option<f32>
pub fn get_keyframe_progress(&self, id: KeyframeId) -> Option<f32>
Get keyframe animation progress (0.0 to 1.0)
Sourcepub fn is_keyframe_playing(&self, id: KeyframeId) -> bool
pub fn is_keyframe_playing(&self, id: KeyframeId) -> bool
Check if keyframe animation is playing
Sourcepub fn start_keyframe(&self, id: KeyframeId)
pub fn start_keyframe(&self, id: KeyframeId)
Start a keyframe animation
Sourcepub fn stop_keyframe(&self, id: KeyframeId)
pub fn stop_keyframe(&self, id: KeyframeId)
Stop a keyframe animation
Sourcepub fn remove_keyframe(&self, id: KeyframeId)
pub fn remove_keyframe(&self, id: KeyframeId)
Remove a keyframe animation
Sourcepub fn register_timeline(&self, timeline: Timeline) -> Option<TimelineId>
pub fn register_timeline(&self, timeline: Timeline) -> Option<TimelineId>
Register a timeline and return its ID
Sourcepub fn is_timeline_playing(&self, id: TimelineId) -> bool
pub fn is_timeline_playing(&self, id: TimelineId) -> bool
Check if timeline is playing
Sourcepub fn start_timeline(&self, id: TimelineId)
pub fn start_timeline(&self, id: TimelineId)
Start a timeline
Sourcepub fn stop_timeline(&self, id: TimelineId)
pub fn stop_timeline(&self, id: TimelineId)
Stop a timeline
Sourcepub fn remove_timeline(&self, id: TimelineId)
pub fn remove_timeline(&self, id: TimelineId)
Remove a timeline
Sourcepub fn with_timeline<F, R>(&self, id: TimelineId, f: F) -> Option<R>
pub fn with_timeline<F, R>(&self, id: TimelineId, f: F) -> Option<R>
Access a timeline to add entries or get values
The closure receives a mutable reference to the timeline. Returns None if the scheduler is dropped or timeline doesn’t exist.
Sourcepub fn register_tick_callback<F>(&self, callback: F) -> Option<TickCallbackId>
pub fn register_tick_callback<F>(&self, callback: F) -> Option<TickCallbackId>
Register a tick callback that runs each frame
The callback receives delta time in seconds. Use this to integrate custom systems (like ECS) with the animation scheduler’s background thread.
Returns None if the scheduler has been dropped.
Example
let world = Arc::new(Mutex::new(World::new()));
let world_clone = world.clone();
let id = handle.register_tick_callback(move |dt| {
let mut world = world_clone.lock().unwrap();
// Run ECS systems with delta time
world.run_systems(dt);
});Sourcepub fn remove_tick_callback(&self, id: TickCallbackId)
pub fn remove_tick_callback(&self, id: TickCallbackId)
Remove a tick callback