pub struct ToastBuilder { /* private fields */ }Expand description
Builder for toast overlays
Implementations
Sourceimpl ToastBuilder
impl ToastBuilder
Sourcepub fn duration_ms(self, ms: u32) -> ToastBuilder
pub fn duration_ms(self, ms: u32) -> ToastBuilder
Set auto-dismiss duration in milliseconds
Sourcepub fn corner(self, corner: Corner) -> ToastBuilder
pub fn corner(self, corner: Corner) -> ToastBuilder
Set corner position
Sourcepub fn content<F>(self, f: F) -> ToastBuilder
pub fn content<F>(self, f: F) -> ToastBuilder
Set the content using a builder function
Sourcepub fn motion_key(self, key: impl Into<String>) -> ToastBuilder
pub fn motion_key(self, key: impl Into<String>) -> ToastBuilder
Set the motion key for triggering content exit animations
When the overlay transitions to Closing state, it will automatically
trigger exit animation on the motion with this key. Use the same key
with motion_derived(key) in your content builder.
Sourcepub fn show(self) -> OverlayHandle
pub fn show(self) -> OverlayHandle
Show the toast